﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;
using Control.STK;
using DBAccess.EAI;
using Boundary.EAI;

namespace Boundary.STK
{
    public partial class Frm433Setup : Form
    {
        int m_nIdle;
        int m_nFlag;   //1 注册频道 2 注销频道 3 系统号

        byte[] pszSerial = new byte[100];       //机器序列号全局变量,作用是判断接受数据是否重复,每个字节代表一个频道(0~99)


       // private ClsCOMMonitorControl objControl = new ClsCOMMonitorControl();
        private int m_nStartFlag = 0;
        private bool isLoad = false;
        private ClsFormProcess2 pClsFormProcess;

        public Frm433Setup()
        {
            InitializeComponent();
            InitializeCbo();
            if (pClsFormProcess == null)
                pClsFormProcess = new ClsFormProcess2(this, true);
            this.cmbdefaultComPort.SelectedIndex = this.cmbdefaultComPort.Items.Count - 1;
        }

        private void InitializeCbo()
        {
            this.cmbdefaultComPort.Items.Clear();

            string[] portnames = SerialPort.GetPortNames();
            for (int i = 0; i < portnames.Length; i++)
            {
                cmbdefaultComPort.Items.Add(portnames[i]);
            }
        }

        private void InitializeSerialPort()
        {
            serialPort1.DataBits = 8;
            serialPort1.DiscardNull = false;
            serialPort1.DtrEnable = false;
            serialPort1.Handshake = Handshake.None;
            serialPort1.Parity = Parity.None;
            serialPort1.ParityReplace = Convert.ToByte("63");
            serialPort1.RtsEnable = false;
            serialPort1.StopBits = StopBits.One;

            serialPort1.PortName = cmbdefaultComPort.Text;
            serialPort1.BaudRate = ClsGlobal.defaultBaudRate;
        }

        private void textBox1_Leave(object sender, EventArgs e)
        {
            if(textBox1.Text.Length > 0)
            {
                int nNum;
                try
                {
                    nNum = Convert.ToInt32(textBox1.Text);
                }
                catch
                {
                    //MessageBox.Show("频道号设置非法！请重新设置。","错误提示",MessageBoxButtons.OK);
                    ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsErrorChannel);
                    textBox1.Text = "";
                    textBox1.Focus();
                    return;
                }
                if((nNum < 0) || (nNum > 99))
                {
                    //MessageBox.Show("频道号设置非法！请重新设置。", "错误提示", MessageBoxButtons.OK);
                    ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsErrorChannel);
                    textBox1.Text = "";
                    textBox1.Focus();
                    return;
                }
            }
        }
        private void textBox2_Leave(object sender, EventArgs e)
        {
            if (textBox2.Text.Length > 0)
            {
                int nNum;
                try
                {
                    nNum = Convert.ToInt32(textBox2.Text);
                }
                catch
                {
                    //MessageBox.Show("系统号设置非法！请重新设置。", "错误提示", MessageBoxButtons.OK);
                    ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsErrorSystemNo);
                    textBox2.Text = "";
                    textBox2.Focus();
                    return;
                }
                if ((nNum < 0) || (nNum > 65536))
                {
                    //MessageBox.Show("系统号设置非法！请重新设置。", "错误提示", MessageBoxButtons.OK);
                    ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsErrorSystemNo);
                    textBox2.Text = "";
                    textBox2.Focus();
                    return;
                }
            }
        }

        private void textBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if((e.KeyChar < 48|| e.KeyChar > 57) && e.KeyChar != 0x08)
            {//非数字与删除不可输入
                e.Handled = true;
            }
        }
        private void textBox2_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < 48 || e.KeyChar > 57) && e.KeyChar != 0x08)
            {//非数字与删除不可输入
                e.Handled = true;
            }
        }

        private void frm433Setup_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Close();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if(0 == textBox1.Text.Length)
            {
                ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsInputChannel);
                //MessageBox.Show("请先输入频道号！", "错误提示", MessageBoxButtons.OK);
                return;
            }
            if (serialPort1 != null && serialPort1.IsOpen)
                serialPort1.Close();
            serialPort1.RtsEnable = false;
            InitializeSerialPort();

            try
            {
                serialPort1.Open();
            }
            catch
            {
                ClsFormInfo.ShowError(this, TOTA.Properties.Resources.rsOpenComError2);
//                MessageBox.Show("无法打开通讯端口！", "错误提示", MessageBoxButtons.OK);
                return;
            }     
            
            m_nIdle = 0; 
            //timer1.Enabled = true;
            
           
            Byte[] SendData = new byte[5];
            SendData[0] = 0xfd;
            SendData[1] = 0xfc;
            
            
          
        
            if(radioButton1.Checked || null==sender)
            {
                SendData[2] = ClsCOMMonitorControl.SHTP_REGCHANNEL;// 0x03;
                //if(0 != GetChannelCode(0, Convert.ToInt32(textBox1.Text), pszBufferOut))
                //{
                //    MessageBox.Show("获取注册频道命令出错！", "错误提示", MessageBoxButtons.OK);
                //    return;
                //}
            }
            else
            {
                SendData[2] = ClsCOMMonitorControl.SHTP_CANCELCHANNEL;// 0x04;
                //if(0 != GetChannelCode(1, Convert.ToInt32(textBox1.Text), pszBufferOut))
                //{
                //    MessageBox.Show("获取注销频道命令出错！", "错误提示", MessageBoxButtons.OK);
                //    return;
                //}
            }
            int num = Convert.ToInt32(textBox1.Text);
            SendData[3] = (byte)(num / 256);
            SendData[4] = (byte)(num % 256);


            if (radioButton1.Checked || null==sender)
            {
                toolStripStatusLabel1.Text = TOTA.Properties.Resources.rsRegistering;// "正在注册频道...";
                m_nFlag = 1;
            }
            else             
            {
                toolStripStatusLabel1.Text = TOTA.Properties.Resources.rsCanceling;// "正在注销频道...";
                m_nFlag = 2;
            }

            m_nStartFlag = 0;
            for (int i = 0; i < 3; i++)
            {
                if (m_nStartFlag > 0)
                    break;
                this.serialPort1.Write(SendData, 0, 5);
                Thread.Sleep(300);
                Application.DoEvents();
                
            }

            if (m_nStartFlag == 0)
            {
                if (radioButton2.Checked && (num == 0)) //因为接收器没有回传信息所以加上这一段
                {
                    Thread.Sleep(200);
                    radioButton1.Checked = true;
                    button1_Click(null, null);
                    return;

                }
                toolStripStatusLabel1.Text = "Ready";
                if (sender!=null)                
                    ClsFormInfo.ShowInformation(this, TOTA.Properties.Resources.rsSetupFail);
                
            }
            else
            {
                toolStripStatusLabel1.Text = "OK";
            }

            //this.serialPort1.Write(SendData, 0, 5);



        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (0 == textBox2.Text.Length)
            {
                //MessageBox.Show("请先输入系统号！", "错误提示", MessageBoxButtons.OK);
                ClsFormInfo.ShowWarning(this, TOTA.Properties.Resources.rsInputSystemNo);
                return;
            }
            if (serialPort1 != null && serialPort1.IsOpen) serialPort1.Close();
            serialPort1.RtsEnable = false;
            InitializeSerialPort();

            try
            {
                serialPort1.Open();
            }
            catch
            {
                ClsFormInfo.ShowError(this, TOTA.Properties.Resources.rsOpenComError2);
                //MessageBox.Show("无法打开通讯端口！", "错误提示", MessageBoxButtons.OK);
                return;
            }
            
            m_nIdle = 0;
            timer1.Enabled = true;

            Byte[] SendData = new byte[5];
            SendData[0] = 0xfd;
            SendData[1] = 0xfc;
            SendData[2] = ClsCOMMonitorControl.SHTP_SETUPSYS;// 0x01;
            int num = Convert.ToInt32(textBox2.Text);
            SendData[3] = (byte)(num / 256);
            SendData[4] = (byte)(num % 256);


            toolStripStatusLabel1.Text = TOTA.Properties.Resources.rsChangeSystemNo;// "正在修改系统号...";
            m_nFlag = 3;
            m_nStartFlag = 0;
            //this.serialPort1.Write(SendData, 0, 5);
            for (int i = 0; i < 3; i++)
            {
                if (m_nStartFlag > 0)
                    break;
                this.serialPort1.Write(SendData, 0, 5);
                Thread.Sleep(300);
                Application.DoEvents();
                
            }

            if (m_nStartFlag == 0)
            {
                toolStripStatusLabel1.Text = "Ready";
                if (sender != null)                
                    ClsFormInfo.ShowInformation(this, TOTA.Properties.Resources.rsSetupFail);

            }
            else
            {
                toolStripStatusLabel1.Text = "OK";
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //int nIndex;

            //m_nIdle++;
            //nIndex = toolStripStatusLabel1.Text.IndexOf("...");
            //if (nIndex > 0)
            //{
            //    toolStripStatusLabel1.Text = toolStripStatusLabel1.Text.Substring(1, nIndex + 2) + Convert.ToString(m_nIdle);
            //}
            //else
            //{
            //    toolStripStatusLabel1.Text = toolStripStatusLabel1.Text + Convert.ToString(m_nIdle);
            //}
            //if (m_nIdle > 5)
            //{
            //    timer1.Enabled = false;
            //    toolStripStatusLabel1.Text = "Ready";
            //    ClsFormInfo.ShowInformation(this, TOTA.Properties.Resources.rsSetupFail);
            //    //MessageBox.Show("设置失败！请检查接收器是否已插入且端口号选择正确。", "信息提示", MessageBoxButtons.OK);
            //}
        }

        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                byte[] pszBufferIn = new byte[1024];
                Array.Clear(pszBufferIn, 0x00, pszBufferIn.Length);

                int nBytesCount;
                while (true)
                {
                    nBytesCount = serialPort1.BytesToRead;
                    Thread.Sleep(2);   //必要延时,以保证整包数据完整接收
                    if (serialPort1.BytesToRead == nBytesCount)
                    {
                        break;
                    }
                }

                if (nBytesCount >= 1024)
                {//缓冲区的数据大于分配的最大内存(协议规定的每包数据的最大值),放弃.
                    serialPort1.Read(pszBufferIn, 0, 1024);
                    return;
                }
                if (nBytesCount != serialPort1.Read(pszBufferIn, 0, nBytesCount))
                {
                    return;
                }

            //    byte[] pszBufferOut = new byte[1024];

                //int nResult = objControl.CheckData(pszBufferIn, nBytesCount);

                //switch (nResult)
                //{

                //    case 1: SetTextBox(pszBufferIn);
                //        break;//系统号应答
                //    case 3: if (1 == m_nFlag)
                //        {
                //            AlterlistView(pszBufferIn);
                //        }
                //        break;//命令包应答
                //    case 4: if (2 == m_nFlag)
                //        {
                //            AlterlistView(pszBufferIn);
                //        }//频道应答
                //        break;

                //    default: break;
                //}
                int nResult = clsRealTimeIntf.CheckData(pszBufferIn, nBytesCount);

                switch ((byte)nResult)
                {

                   
                    case ClsCOMMonitorControl.SHTP_SETUPSYS:
                        SetTextBox(pszBufferIn);
                        m_nStartFlag++; 
                        break;//系统号应答
                    case ClsCOMMonitorControl.SHTP_REGCHANNEL:
                        AlterlistView(pszBufferIn);
                        //if (1 == m_nFlag)
                        //{
                        //    AlterlistView(pszBufferIn);
                        //}
                        m_nStartFlag++; 
                        break;//命令包应答
                    case ClsCOMMonitorControl.SHTP_CANCELCHANNEL:
                        AlterlistView(pszBufferIn);
                        //if (2 == m_nFlag)
                        //{
                        //    AlterlistView(pszBufferIn);
                        //}//频道应答
                        m_nStartFlag++; 
                        break;

                    default: break;
                }
                timer1.Enabled = false;
                toolStripStatusLabel1.Text = TOTA.Properties.Resources.rsSetupSucess;// "设置成功";
                //serialPort1.Close();
                /// 0x01	设置系统号
                /// 0x02	设置发射功率
                /// 0x03	注册信道
                /// 0x04	删除信道
            }
            catch (Exception ee)
            {
                ClsFormInfo.ShowError(this, "serialPort1_DataReceivedErr!\n" + ee.Message);
                
            } 
        }
        private delegate void textBoxCallback(byte[] data);
        public void SetTextBox(byte[] data)
        {
            try
            {
                if (this.textBox2.InvokeRequired)
                {
                    textBoxCallback d = new textBoxCallback(SetTextBox);
                    this.Invoke(d, new object[] { data });
                }
                else
                {
                    textBox2.Text = Convert.ToString(data[3]*256 + data[4]);
                }
            }
            catch (Exception e)
            {

                ClsFormInfo.ShowError(this, e.Message);
            }
            m_nIdle = 0;

        }

        private delegate void listViewCallback(byte[] data);
        public void AlterlistView(byte[] data)
        {
            try
            {
                if (this.listView1.InvokeRequired)
                {
                    listViewCallback d = new listViewCallback(AlterlistView);
                    this.Invoke(d, new object[] { data });
                }
                else
                {
                    if ("0" == textBox1.Text)
                    {//查询已注册频道 
                        listView1.Items.Clear();
                        for (int i = 3; i < data.Length; i++)
                        {
                            if (data[i] == 0)
                            {
                                break;
                            }
                            ListViewItem listtiem = new ListViewItem();
                            listtiem.SubItems[0].Text = Convert.ToString(listView1.Items.Count + 1);
                            listtiem.SubItems.Add(Convert.ToString(data[i]));
                            this.listView1.Items.Add(listtiem);
                        }
                    }
                    else
                    {//注册时增加,注销时为删除
                        if (radioButton1.Checked)
                        {
                            bool bFlag = true;
                            for (int i = 0; i < listView1.Items.Count; i++)
                            {
                                if (textBox1.Text == listView1.Items[i].SubItems[1].Text)
                                {
                                    bFlag = false;
                                    break;
                                }
                            }
                            if (bFlag)
                            {
                                ListViewItem listtiem = new ListViewItem();
                                listtiem.SubItems[0].Text = Convert.ToString(listView1.Items.Count + 1);
                                listtiem.SubItems.Add(textBox1.Text);
                                this.listView1.Items.Add(listtiem);
                            }
                        }
                        else 
                        {
                            for (int i = 0; i < listView1.Items.Count; i++)
                            {
                                if (textBox1.Text == listView1.Items[i].SubItems[1].Text)
                                {
                                    ListViewItem listtiem = new ListViewItem();
                                    listtiem = listView1.Items[i];
                                    listView1.Items.Remove(listtiem);
                                    //修改余下的顺序号
                                    for (int j = i; j < listView1.Items.Count; j++)
                                    {
                                        listView1.Items[j].SubItems[0].Text = Convert.ToString(j+1);
                                    }
                                    break;
                                }
                            }
                        }
                    }
                }
                m_nIdle = 0;
            }
            catch (Exception e)
            {
                ClsFormInfo.ShowError(this, e.Message);
                
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void Frm433Setup_Load(object sender, EventArgs e)
        {
            //isLoad = true;
            cmbdefaultComPort.SelectedIndex = cmbdefaultComPort.Items.IndexOf(ClsGlobal.defaultComPort);
            //isLoad = false;
            
        }

        private void cboCom_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbdefaultComPort.SelectedIndex < 0)
                return;
            button1_Click(null, null);
            button2_Click(null, null);
        }

        private void FillComPort()
        {
            cmbdefaultComPort.Items.Clear();

            string[] portnames = System.IO.Ports.SerialPort.GetPortNames();
            for (int i = 0; i < portnames.Length; i++)
            {
                cmbdefaultComPort.Items.Add(portnames[i]);
            }
            if (cmbdefaultComPort.Items.Count == 1)
                cmbdefaultComPort.SelectedIndex = 0;
        }

        private void cboCom_Enter(object sender, EventArgs e)
        {
            //FillComPort();
        }
    }
}